import functools
import pathlib
from typing import Any

import ijson

DATAHUB_CURRENCY_RESOURCE_PATH = (
    pathlib.Path(__file__).resolve().parent / "data" / "currencies.json"
)


class CurrencyNotFoundError(Exception):
    pass


class Currency:
    def __init__(
        self,
        alphabetical_code,
        currency_name,
        entity,
        decimal_places,
        numeric_code,
        withdrawal_date,
    ):
        self.withdrawal_date = withdrawal_date
        self.numeric_code = numeric_code
        self.decimal_places = decimal_places
        self.entity = entity
        self.currency_name = currency_name
        self.alphabetical_code = alphabetical_code

    @classmethod
    def from_dict(cls, d):
        numeric_code = d["NumericCode"]
        if numeric_code is not None:
            numeric_code = int(numeric_code)

        decimal_places = d["MinorUnit"]
        if isinstance(decimal_places, str) and decimal_places.isdigit():
            decimal_places = int(decimal_places)

        return cls(
            alphabetical_code=d["AlphabeticCode"],
            currency_name=d["Currency"],
            entity=d["Entity"],
            decimal_places=decimal_places,
            numeric_code=numeric_code,
            withdrawal_date=d["WithdrawalDate"],
        )

    def __str__(self):
        if self.was_withdrawn:
            return "{0} (not active) - withdrawn at {1}".format(
                self.currency_name, self.withdrawal_date
            )

        return "{0} {1}".format(self.currency_name, self.numeric_code)

    def __repr__(self):
        return "<Currency {0}>".format(self.currency_name)

    @property
    def was_withdrawn(self):
        return self.withdrawal_date is not None


@functools.lru_cache(maxsize=512)
def find_currency(**filters: Any):
    for currency in iter_currency():
        for filter_key, filter_value in filters.items():
            f = getattr(currency, filter_key, None)
            if f is None:
                continue
            if f == filter_value:
                return currency
            elif isinstance(f, str) and f.lower() == filter_value:
                return currency

    raise CurrencyNotFoundError(f"Currency by {filters} filters was not found")


def iter_currency():
    with open(DATAHUB_CURRENCY_RESOURCE_PATH, "r") as f:
        for currency_dict in ijson.items(f, "item", use_float=True):
            yield Currency.from_dict(currency_dict)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['iso4217']

package_data = \
{'': ['*'], 'iso4217': ['data/*']}

modules = \
['__init__']
install_requires = \
['ijson>=3.1.4,<4.0.0']

setup_kwargs = {
    'name': 'python-iso4217',
    'version': '0.0.4',
    'description': 'Lightweight ISO 4217 currency package',
    'long_description': '# `python-iso4217`: fast currency data package for Python\n\n---\n\nThis python package automatically updates its own\ncurrency data using github actions and immediately release\nnew version of package with updates. So it\'s super fast and at the same time always contains fresh data.\n\n## Installation\n\n### Pip\n\n```bash\npip install python-iso4217\n```\n\n### Poetry\n\n```bash\npoetry add python-iso4217\n```\n\n## Usage\n\n```python\nfrom iso4217 import find_currency, iter_currency\n\n# Filters that could be applied:\n#   * currency_name\n#   * alphabetical_code\n#   * entity\n#   * decimal_places\n#   * numeric_code\n#   * withdrawal_date\n# The filters correspond to attributes of `Currency` class\n\n# accept arbitrary filters as key/value pairs\ncurrency = find_currency(currency_name="usd")\n\n# if currency was not found, the lib will immediately raise CurrencyNotFoundError\ndefunct_currency = find_currency(currency_name="abc123")\n\n# You need generator to lazy iter currencies? Ok\n\nfor c in iter_currency():\n# do something\n```\n\n## Advanced\n\n\n### Work with pydantic\n\n```python\nimport inspect\n\nfrom iso4217 import Currency as _Currency, find_currency\nfrom pydantic import BaseModel, root_validator, StrictInt\n\n\nclass Currency(BaseModel, _Currency):\n    __root__: StrictInt\n\n    @root_validator(skip_on_failure=True)\n    def humanize(cls, values):\n        currency_numeric_code: int = values.get("__root__")\n        # inspect.getfullargspec(_Currency.__init__) will contain `self`, so we cut it off\n        orig_currency_cls_dunder_init_arg_names = inspect.getfullargspec(_Currency.__init__).args[1:]\n        currency = find_currency(numeric_code=currency_numeric_code)\n        return {\n            k: getattr(currency, k, None)\n            for k in orig_currency_cls_dunder_init_arg_names\n        }\n```',
    'author': 'GLEF1X',
    'author_email': 'glebgar567@gmail.com',
    'maintainer': 'GLEF1X',
    'maintainer_email': 'glebgar567@gmail.com',
    'url': 'https://github.com/GLEF1X/python-iso4217',
    'packages': packages,
    'package_data': package_data,
    'py_modules': modules,
    'install_requires': install_requires,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)



import pathlib
from typing import Any, Dict, Generator, Optional

DATAHUB_CURRENCY_RESOURCE_PATH: pathlib.Path

class CurrencyNotFoundError(Exception): ...

class Currency:
    alphabetical_code: str
    currency_name: str
    entity: str
    decimal_places: int
    numeric_code: int
    withdrawal_date: Optional[str] = None

    def __init__(
        self,
        alphabetical_code: str,
        currency_name: str,
        entity: str,
        decimal_places: int,
        numeric_code: int,
        withdrawal_date: Optional[str] = None,
    ) -> None: ...
    @classmethod
    def from_dict(cls, d: Dict[str, Any]) -> "Currency": ...
    @property
    def was_withdrawn(self) -> bool: ...
    def __str__(self) -> str: ...
    def __repr__(self) -> str: ...

def find_currency(**filters: Any) -> Currency: ...
def iter_currency() -> Generator[Currency, None, None]: ...
